/* Autogenerated Changes here will be lost!
 * generated by ../gen_sw.pl Mon Apr 11 16:10:18 2005 iop_sw.cfg
 */
#define regi_iop_version (regi_iop + 0)
#define regi_iop_fifo_in0_extra (regi_iop + 64)
#define regi_iop_fifo_in1_extra (regi_iop + 128)
#define regi_iop_fifo_out0_extra (regi_iop + 192)
#define regi_iop_fifo_out1_extra (regi_iop + 256)
#define regi_iop_trigger_grp0 (regi_iop + 320)
#define regi_iop_trigger_grp1 (regi_iop + 384)
#define regi_iop_trigger_grp2 (regi_iop + 448)
#define regi_iop_trigger_grp3 (regi_iop + 512)
#define regi_iop_trigger_grp4 (regi_iop + 576)
#define regi_iop_trigger_grp5 (regi_iop + 640)
#define regi_iop_trigger_grp6 (regi_iop + 704)
#define regi_iop_trigger_grp7 (regi_iop + 768)
#define regi_iop_crc_par0 (regi_iop + 896)
#define regi_iop_crc_par1 (regi_iop + 1024)
#define regi_iop_dmc_in0 (regi_iop + 1152)
#define regi_iop_dmc_in1 (regi_iop + 1280)
#define regi_iop_dmc_out0 (regi_iop + 1408)
#define regi_iop_dmc_out1 (regi_iop + 1536)
#define regi_iop_fifo_in0 (regi_iop + 1664)
#define regi_iop_fifo_in1 (regi_iop + 1792)
#define regi_iop_fifo_out0 (regi_iop + 1920)
#define regi_iop_fifo_out1 (regi_iop + 2048)
#define regi_iop_scrc_in0 (regi_iop + 2176)
#define regi_iop_scrc_in1 (regi_iop + 2304)
#define regi_iop_scrc_out0 (regi_iop + 2432)
#define regi_iop_scrc_out1 (regi_iop + 2560)
#define regi_iop_timer_grp0 (regi_iop + 2688)
#define regi_iop_timer_grp1 (regi_iop + 2816)
#define regi_iop_timer_grp2 (regi_iop + 2944)
#define regi_iop_timer_grp3 (regi_iop + 3072)
#define regi_iop_sap_in (regi_iop + 3328)
#define regi_iop_sap_out (regi_iop + 3584)
#define regi_iop_spu0 (regi_iop + 3840)
#define regi_iop_spu1 (regi_iop + 4096)
#define regi_iop_sw_cfg (regi_iop + 4352)
#define regi_iop_sw_cpu (regi_iop + 4608)
#define regi_iop_sw_mpu (regi_iop + 4864)
#define regi_iop_sw_spu0 (regi_iop + 5120)
#define regi_iop_sw_spu1 (regi_iop + 5376)
#define regi_iop_mpu (regi_iop + 5632)
